con
  _clkmode = xtal1 + pll16x      'set to ext low speed crystal
  _xinfreq = 5_000_000

e_to_2 = 1.44269504088896
lfsr_scale = pi/float(posx)
lfsr_scale2 = 32.0/float(posx)
lfsr_scale3 = 126.0/float(posx)

obj

  S : "parallax serial terminal" 'serial communications with the host.  
  F32 : "F32_1_6_pre"           'single cog assembly floating point lib
  Fe : "FME"      'My upgrade to FloatMath to include Log, (exp?) (and optionally Sin, Cos, Tan and inverses)
  Fi : "Float32Full"            'two cog assembly floating point lib
  Q : "spin_trigpack_log"       'fixed point Qs15.16 math library.
  Fs : "FloatString"            'conversion utilities between strings and floats

var
  long  sa
  long  xa
  long  ma
  long  seed
  byte  str_buf[100]

pub main 
'' do nothing for now

  'start the serial port
  S.start(115200)

  'start F32
  F32.start

  'start Float32Full
  Fi.start

  'initialise FloatMathExtended
  'nop

  'initialise fixed point math lib
  Q.start_driver

  'initilise LFSR seed with official random number.  
  seed := $A1E24BAA             'to get a new seed execute "dec2hex(round(abs(rand*2^32)), 8)" in matlab.
  sprout := $F878C4BF
  
  'start a test of the fixed point log2, anti-log2
  'Qpoint_test

  'start a test of FloatMathExtended
  'first test the new Fcmp
  'Fcmp_test
  'Finteger32_test
  'Fsin_test
  'Fcos_test
  'Ftan_test
  'Ftrig_error_stats

  'Fatan2_test
  'Fatan2_error_stats
  'Fasin_acos_test
  'frac_test
  'exp2_test
  'exp2_error_stats
  'exp2_error_raw
  'log2_test
  'log2_error_raw
  'expN_test
  'logN_test

  'Fatan2_error_raw
  Ftan_error_raw



pub Ftan_error_raw | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, and FloatMathExtended version of log2()
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("input, F32.tan[rad], Fe.tan[rad]" ))
  S.char(S#NL)


  repeat

    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    singleA := F32.Fmul(SingleA, lfsr_scale)            'scale to random number to the -pi to pi range
    'singleA := F32.exp2(singleA)

    'S.Str(Fs.FloatToScientific(singleA))
    S.hex(singleA,8)
    S.char(",")
    'F32
    temp := F32.tan(singleA)
    'S.Str(Fs.FloatToScientific(temp))
    S.hex(temp,8)
    S.char(",")

    temp := Fe.tan(singleA)
    'S.Str(Fs.FloatToScientific(temp))
    S.hex(temp,8)

    S.char(S#NL)
    waitcnt(cnt + clkfreq/300)



pub Fatan2_error_raw | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, Float32full, and FloatMathExtended versions of sin, cos, and tan
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.
'collums:
'angle, F32.sin, F32.cos, F32.tan, Fi.sin, Fi.cos, Fi.tan, Fe.sin, Fe.cos, Fe.tan

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("x, y, F32.atna2[rad], Fi.atan2[rad], Fe.atan2[rad]" ))
  S.char(S#NL)


  repeat
    '?seed                       'get next psuedo-random number
    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    'singleA := F32.Fmul(SingleA, lfsr_scale)            'scale to random number to the -pi to pi range

    '?seed
    seed := random(seed)
    singleB := F32.FFloat(seed)

    'record inputs to test
    'S.Str(Fs.FloatToScientific(singleA))                'output the y-axis vector length
    S.hex(singleA,8)
    S.char(",")
    'S.Str(Fs.FloatToScientific(singleB))                'output the x-axis vector length
    S.hex(singleB,8)
    S.char(",")
    'F32
    temp := F32.atan2(singleA, singleB)
    'S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.hex(temp,8)
    S.char(",")
    'Float32full
    temp := Fi.atan2(singleA, singleB)
    'S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.hex(temp,8)
    S.char(",")
    'FloatMathExtended
    temp := Fe.atan2(singleA, singleB)
    'S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.hex(temp,8)
    'S.char(",")
    S.char(S#NL)
    waitcnt(cnt + clkfreq/300)




pub logN_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare LogB(pi,b) results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    'S.Str(@str_buf)            
    singleA := Fs.StringToFloat(@str_buf)
    S.str(Fs.FloatToString(singleA))
    
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)

    S.str(String("  F32: "))
    temp := F32.log10(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.logB(pi,singleA)
    S.str(Fs.FloatToString(temp))
    S.char(",")
    S.dec(fe.get_debug)
    S.char(",")
    S.dec(fe.get_debug2)

    'S.str(string("  Ffull: "))
    'temp := Fi.exp2(singleA)
    'S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)




pub expN_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare pow(pi,A) results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    'S.Str(@str_buf)            
    singleA := Fs.StringToFloat(@str_buf)
    S.str(Fs.FloatToString(singleA))
    
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)

    S.str(String("  F32: "))
    temp := F32.pow(pi ,singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.pow(pi ,singleA)
    S.str(Fs.FloatToString(temp))
    S.char(",")
    S.dec(fe.get_debug)

    'S.str(string("  Ffull: "))
    'temp := Fi.exp2(singleA)
    'S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)


pub log2_error_raw | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, and FloatMathExtended version of log2()
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("input, F32.log2[rad], Fe.log2[rad]" ))
  S.char(S#NL)


  repeat

    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    singleA := F32.Fmul(SingleA, lfsr_scale3)            'scale to random number to the -126 to 126 range
    singleA := F32.exp2(singleA)

    'S.Str(Fs.FloatToScientific(singleA))
    S.hex(singleA,8)
    S.char(",")
    'F32
    temp := F32.log2(singleA)
    'S.Str(Fs.FloatToScientific(temp))
    S.hex(temp,8)
    S.char(",")

    temp := Fe.log2(singleA)
    'S.Str(Fs.FloatToScientific(temp))
    S.hex(temp,8)

    S.char(S#NL)
    waitcnt(cnt + clkfreq/300)



pub log2_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare Log2 results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    'S.Str(@str_buf)            
    singleA := Fs.StringToFloat(@str_buf)
    S.str(Fs.FloatToString(singleA))
    
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)

    S.str(String("  F32: "))
    temp := F32.log2(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.log2(singleA)
    S.str(Fs.FloatToString(temp))
    S.char(",")
    S.dec(fe.get_debug)
    S.char(",")
    S.dec(fe.get_debug2)

    'S.str(string("  Ffull: "))
    'temp := Fi.exp2(singleA)
    'S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)




pub exp2_error_raw | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, and FloatMathExtended version of exp2()
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("input, F32.exp2[rad], Fe.exp2[rad]" ))
  S.char(S#NL)


  repeat
    '?seed                       'get next psuedo-random number
    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    singleA := F32.Fmul(SingleA, lfsr_scale3)            'scale to random number to the -32 to 32 range

    '?seed
    'seed := random(seed)
    'singleB := F32.FFloat(seed)

    'record inputs to test
    'S.Str(Fs.FloatToScientific(singleA))                'output the y-axis vector length
    S.hex(singleA,8)
    S.char(",")
    'F32
    temp := F32.exp2(singleA)
    'S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.hex(temp,8)
    S.char(",")
    'Float32full
    {temp := Fi.atan2(singleA, singleB)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.char(",") '}
    'FloatMathExtended
    temp := Fe.exp2(singleA)
    'S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.hex(temp,8)
    'S.char(",")
    S.char(S#NL)
    waitcnt(cnt + clkfreq/300)

pub exp2_error_stats | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, and FloatMathExtended version of exp2()
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("input, F32.exp2[rad], Fe.exp2[rad]" ))
  S.char(S#NL)


  repeat
    '?seed                       'get next psuedo-random number
    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    singleA := F32.Fmul(SingleA, lfsr_scale2)            'scale to random number to the -32 to 32 range

    '?seed
    'seed := random(seed)
    'singleB := F32.FFloat(seed)

    'record inputs to test
    S.Str(Fs.FloatToScientific(singleA))                'output the y-axis vector length
    S.char(",")
    'F32
    temp := F32.exp2(singleA)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.char(",")
    'Float32full
    {temp := Fi.atan2(singleA, singleB)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.char(",") '}
    'FloatMathExtended
    temp := Fe.exp2(singleA)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    'S.char(",")
    S.char(S#NL)
    waitcnt(cnt + clkfreq/1000)

  

pub exp2_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare Exp2 results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    'S.Str(@str_buf)            
    singleA := Fs.StringToFloat(@str_buf)
    S.str(Fs.FloatToString(singleA))
    
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)

    S.str(String("  F32: "))
    temp := F32.exp2(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.exp2(singleA)
    S.str(Fs.FloatToString(temp))
    S.char(",")
    S.dec(fe.get_debug)

    'S.str(string("  Ffull: "))
    'temp := Fi.exp2(singleA)
    'S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)

  

con
mantissa_scale_float = 1.0/float(1<<29)

pub frac_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare frac results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    'S.Str(@str_buf)            
    singleA := Fs.StringToFloat(@str_buf)
    S.str(Fs.FloatToString(singleA))
    
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
   
    S.str(string("  int: "))
    'temp := F32.FloatTrunc(singleA)
    temp := fe.ffloat(fe.Ftrunc(singleA))
    S.str(Fs.FloatToString(temp))
    
    S.str(String("  F32: "))
    temp := F32.frac(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.frac(singleA)
    S.str(Fs.FloatToString(temp))

    'S.str(string("  FMEl: "))
    'temp := Fe.frac_long(singleA)
    'S.str(Fs.FloatToString(temp))

    {S.str(string("  FMEi: "))
    temp := Fe.frac_int(singleA)
    S.dec(temp)
    s.char(",")
    temp := Fe.Fmul(Fe.FFloat(temp), mantissa_scale_float)
    S.str(Fs.FloatToString(temp))'}

    S.str(string("  Ffull: "))
    temp := Fi.frac(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)


var
  long sprout

pub random(spud)
''add some unsteady clocking to the LFSR to improve it's quality a bit
  repeat (((spud <<13) >> 30)+1)                        'run the LFSR 1-4 times bassed on center bits of spud.
    ?sprout
  repeat (((sprout <<23) >> 30)+1)                      'run the LFSR 1-4 times bassed on center bits of sprout.  
    ?spud
  result := spud



pub Fasin_acos_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare asin and acos results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    
    '
    {S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)'}
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    'S.str(string("  B: "))
    'S.hex(singleB,8)
    
    {Fe.unpack(@sa,F32.Fmul(singleA,Fe#rad_to_subcor))
    S.str(string("sign "))
    S.dec(sa)
    S.str(string(" exponent "))
    S.dec(xa)
    S.str(string(" Mantissia "))
    S.hex(ma,8)'}
    
    S.str(String("  F32: "))
    temp := F32.acos(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.acos(singleA)
    S.str(Fs.FloatToString(temp))

    S.str(string("  Ffull: "))
    temp := Fi.acos(singleA)
    S.str(Fs.FloatToString(temp))

    'asin
    S.str(String("  F32: "))
    temp := F32.asin(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.asin(singleA)
    S.str(Fs.FloatToString(temp))

    S.str(string("  Ffull: "))
    temp := Fi.asin(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)




pub Fatan2_error_stats | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, Float32full, and FloatMathExtended versions of sin, cos, and tan
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.
'collums:
'angle, F32.sin, F32.cos, F32.tan, Fi.sin, Fi.cos, Fi.tan, Fe.sin, Fe.cos, Fe.tan

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("x, y, F32.atna2[rad], Fi.atan2[rad], Fe.atan2[rad]" ))
  S.char(S#NL)


  repeat
    '?seed                       'get next psuedo-random number
    seed := random(seed)
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    'singleA := F32.Fmul(SingleA, lfsr_scale)            'scale to random number to the -pi to pi range

    '?seed
    seed := random(seed)
    singleB := F32.FFloat(seed)

    'record inputs to test
    S.Str(Fs.FloatToScientific(singleA))                'output the y-axis vector length
    S.char(",")
    S.Str(Fs.FloatToScientific(singleB))                'output the x-axis vector length
    S.char(",")
    'F32
    temp := F32.atan2(singleA, singleB)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.char(",")
    'Float32full
    temp := Fi.atan2(singleA, singleB)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    S.char(",")
    'FloatMathExtended
    temp := Fe.atan2(singleA, singleB)
    S.Str(Fs.FloatToScientific(temp))                'output the angle
    'S.char(",")
    S.char(S#NL)
    waitcnt(cnt + clkfreq/1000)



pub Fatan2_test | singleA, singleB, temp
'interactively test the cordic Atan2 function versus the obex codes.
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type in two numbers to compare Fcmp results"))
  S.char(S#NL)

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    S.str(string("  B: "))
    S.hex(singleB,8)

    S.char(",")
    temp := F32.atan2(singleA, singleB)
    S.str(Fs.FloatToString(temp))
    S.char(",")

    temp := Fe.atan2(singleA, singleB)
    S.str(Fs.FloatToString(temp))
    S.char(",")

    temp := Fi.atan2(singleA, singleB)
    S.str(Fs.FloatToString(temp))
    S.char("#")
    S.dec(fe.get_debug)
    S.char("#")
    S.dec(fe.get_debug2)

    S.char(S#NL)



  

pub Ftrig_error_stats | singleA, singleB, temp
'run a psuedo-random number ranging from -pi to pi through F32, Float32full, and FloatMathExtended versions of sin, cos, and tan
'log as a CSV file over the serial port for comparison to Matlab sin, cos, and tan functions.
'use a 9 digit scientific notation format for the numbers.
'collums:
'angle, F32.sin, F32.cos, F32.tan, Fi.sin, Fi.cos, Fi.tan, Fe.sin, Fe.cos, Fe.tan

  waitcnt( cnt + clkfreq*2) 'wait 2 seconds for serial port to be open.
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(string("Collums of CSV output" ))
  S.char(S#NL)
  
  S.str(string("angle [rad], F32.sin, F32.cos, F32.tan, Fi.sin, Fi.cos, Fi.tan, Fe.sin, Fe.cos, Fe.tan" ))
  S.char(S#NL)


  repeat
    ?seed                       'get next psuedo-random number
    singleA := F32.FFloat(seed) 'convert seed to signed floating point
    singleA := F32.Fmul(SingleA, lfsr_scale)            'scale to random number to the -pi to pi range

    'record angle to test
    S.Str(Fs.FloatToScientific(singleA))                'output the angle
    S.char(",")
    'test F32
    singleB := F32.sin(singleA)                         'calculate sin(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := F32.cos(singleA)                         'calculate cos(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := F32.tan(singleA)                         'calculate tan(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    'test Float32full
    singleB := Fi.sin(singleA)                         'calculate sin(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := Fi.cos(singleA)                         'calculate cos(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := Fi.tan(singleA)                         'calculate tan(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    'test FloatMathExtended
    singleB := Fe.sin(singleA)                         'calculate sin(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := Fe.cos(singleA)                         'calculate cos(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    S.char(",")
    singleB := Fe.tan(singleA)                         'calculate tan(singleA)
    S.Str(Fs.FloatToScientific(singleB))                'output the result
    'S.char(",")
    S.char(S#NL)                                        'new line


  
  
  
pub Ftan_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare Fcos results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    
    '
    {S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)'}
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    'S.str(string("  B: "))
    'S.hex(singleB,8)
    
    {Fe.unpack(@sa,F32.Fmul(singleA,Fe#rad_to_subcor))
    S.str(string("sign "))
    S.dec(sa)
    S.str(string(" exponent "))
    S.dec(xa)
    S.str(string(" Mantissia "))
    S.hex(ma,8)'}
    
    S.str(String("  F32: "))
    temp := F32.tan(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.tan(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(",")
    temp := Fe.get_debug
    S.hex(temp,8)
    'S.str(Fs.FloatToString(temp))

    S.str(string("  Ffull: "))
    temp := Fi.tan(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)  

pub Fcos_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number to compare Fcos results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    
    '
    {S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)'}
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    'S.str(string("  B: "))
    'S.hex(singleB,8)
    
    {Fe.unpack(@sa,F32.Fmul(singleA,Fe#rad_to_subcor))
    S.str(string("sign "))
    S.dec(sa)
    S.str(string(" exponent "))
    S.dec(xa)
    S.str(string(" Mantissia "))
    S.hex(ma,8)'}
    
    S.str(String("  F32: "))
    temp := F32.cos(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.cos(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(",")
    temp := Fe.get_debug
    S.hex(temp,8)
    'S.str(Fs.FloatToString(temp))

    S.str(string("  Ffull: "))
    temp := Fi.cos(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)



pub Fsin_test | singleA, singleB, temp
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a number (-pi to pi) to compare Fsin results"))
  S.char(S#NL)  

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    
    '
    {S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)'}
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    'S.str(string("  B: "))
    'S.hex(singleB,8)
    
    {Fe.unpack(@sa,F32.Fmul(singleA,Fe#rad_to_subcor))
    S.str(string("sign "))
    S.dec(sa)
    S.str(string(" exponent "))
    S.dec(xa)
    S.str(string(" Mantissia "))
    S.hex(ma,8)'}
    
    S.str(String("  F32: "))
    temp := F32.sin(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.str(string("  FME: "))
    temp := Fe.sin(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(",")
    temp := Fe.get_debug
    S.hex(temp,8)
    'S.str(Fs.FloatToString(temp))

    S.str(string("  Ffull: "))
    temp := Fi.sin(singleA)
    S.str(Fs.FloatToString(temp))
    
    S.char(S#NL)

  
pub Finteger32_test | singleA, singleB
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type a two numbers to compare Fmod results"))
  S.char(S#NL)

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    
    'Fe.unpack(@sa,singleA)
    S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)'}
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    S.str(string("  B: "))
    S.hex(singleB,8)

    {S.str(string("sign "))
    S.dec(sa)
    S.str(string(" exponent "))
    S.dec(xa)
    S.str(string(" Mantissia "))
    S.hex(ma,8)'}
    S.str(String("  F32: "))
    S.str(Fs.FloatToString(F32.Fmod(singleA,singleB)))
    S.str(string("  FME: "))
    'S.str(Fs.FloatToString(Fe.Fadd(singleA, singleB)))
    S.str(Fs.FloatToString(Fe.Fmod(singleA,singleB)))
    S.char(",")
    S.str(Fs.FloatToString(Fe.Ftrunc(Fe.Fdiv(SingleA, singleB))))
    S.char(",")
    S.str(Fs.FloatToString(Fe.get_debug))
    S.str(string("  Ffull: "))
    S.str(Fs.FloatToString(Fi.Fmod(singleA,singleB)))
    'S.char(",")
    'S.dec(Fe.get_debug)
    S.char(S#NL)


  

pub Fcmp_test | singleA, singleB
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type in two numbers to compare Fcmp results"))
  S.char(S#NL)

  repeat
    S.str(String("type in singleA: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleA := Fs.StringToFloat(@str_buf)
    S.char(S#NL)
    S.str(String("type in singleB: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    singleB := Fs.StringToFloat(@str_buf)
    S.char(S#NL)
    S.str(string("A: "))
    S.hex(singleA,8)
    S.str(string("  B: "))
    S.hex(singleB,8)
    S.str(String("  F32: "))
    S.dec(F32.Fcmp(singleA, singleB))
    S.str(string("  FME: "))
    S.dec(Fe.Fcmp(singleA, singleB))
    S.char(S#NL)




pub Qpoint_test | temp, temp_float, temp_Qv, temp2, mid_float, mid_Qv 
  'wait for PST to get focus
  waitcnt( cnt + clkfreq*3)
  S.str(string("Testing different math libraries" ))
  S.char(S#NL)

  S.str(String("Type in a posative integer to compare Log2 results (enter a negative value to end this step)"))
  S.char(S#NL)

  repeat
    S.str(String("type a value: "))
    S.StrInMax(@str_buf,99)
    S.Str(@str_buf)
    temp_float := Fs.StringToFloat(@str_buf)
    temp_Qv := Q.StrToQval(@str_buf)
    'temp := S.decIn             'get an integer
    'S.dec(temp)
    'if temp < 0
    '  quit
    'temp_float := F32.FFloat(temp)
    'temp_Qv := Q.Qval(temp)
    S.char(S#TB)

    S.str(String("F32: "))
    mid_float := F32.Log2(temp_float)
    S.str(Fs.FloatToString(mid_float))
    S.str(string(" inv: "))
    S.str(Fs.FloatToString(F32.Exp2(mid_float)))
    S.char(S#TB)

    S.str(String("Float32Full: "))
    temp2 := Fi.log(temp_float)
    temp2 := Fi.Fmul(temp2,e_to_2) 'convert to a base 2 logarithm 
    S.str(Fs.FloatToString(temp2))
    S.char(S#TB)
    
    S.str(string("fixed: "))
    mid_Qv := Q.QLog2(temp_QV)
    S.str(Q.QvalToStr(mid_Qv))
    S.str(string(" inv: "))
    S.str(Q.QvalToStr(Q.QAlog2(mid_QV)))
    S.char(",")
    'S.hex(Q.get_debug,8)
    'S.str(Q.QvalToStr(Q.get_debug))
    S.char(S#NL)

  
   